"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolve = exports.reject = exports.findPromiseLink = exports.awaitPromise = exports.delay = void 0;
const debug_1 = __importDefault(require("debug"));
const global_ids_1 = require("./global-ids");
const gql_1 = require("./gql");
const debug = (0, debug_1.default)('deeplinks:promise');
const delay = (time) => new Promise(res => setTimeout(() => res(null), time));
exports.delay = delay;
function awaitPromise(options) {
    const id = options.id;
    const timeout = options.timeout || 1000;
    const client = options.client;
    return new Promise((res, rej) => __awaiter(this, void 0, void 0, function* () {
        var _a, _b, _c, _d;
        debug('promise', { id });
        if (!id) {
            debug('!id', { id: options.id });
            return rej('options.id must be defined!');
        }
        let observing;
        try {
            while (true) {
                const result = yield client.query((0, gql_1.generateQuery)({
                    queries: [(0, gql_1.generateQueryData)({ tableName: 'links', returning: `
          id type { id value }
          `, variables: {
                                where: {
                                    _or: [
                                        { id: { _eq: id } },
                                        { from_id: { _eq: id }, type_id: { _eq: global_ids_1.GLOBAL_ID_THEN } },
                                        {
                                            from: {
                                                type_id: { _eq: global_ids_1.GLOBAL_ID_PROMISE },
                                                in: { type_id: { _eq: global_ids_1.GLOBAL_ID_THEN }, from_id: { _eq: id } },
                                            },
                                            _or: [
                                                { type_id: { _eq: global_ids_1.GLOBAL_ID_RESOLVED } },
                                                { type_id: { _eq: global_ids_1.GLOBAL_ID_REJECTED } },
                                            ],
                                        },
                                    ],
                                },
                            } })],
                    name: 'PROMISE',
                }));
                debug('result', result);
                try {
                    if (result === null || result === void 0 ? void 0 : result.errors) {
                        debug('error', result === null || result === void 0 ? void 0 : result.errors);
                        return rej(result === null || result === void 0 ? void 0 : result.errors);
                    }
                    else if (result === null || result === void 0 ? void 0 : result.data) {
                        debug('data', result.data);
                        const links = (_a = result.data) === null || _a === void 0 ? void 0 : _a.q0;
                        if (links.length) {
                            let thenExists = false;
                            let thenResolved = false;
                            let thenRejected = false;
                            for (let l = 0; l < links.length; l++) {
                                const link = links[l];
                                if (((_b = link === null || link === void 0 ? void 0 : link.type) === null || _b === void 0 ? void 0 : _b.id) === global_ids_1.GLOBAL_ID_THEN)
                                    thenExists = true;
                                else if (((_c = link === null || link === void 0 ? void 0 : link.type) === null || _c === void 0 ? void 0 : _c.id) === global_ids_1.GLOBAL_ID_RESOLVED)
                                    thenResolved = true;
                                else if (((_d = link === null || link === void 0 ? void 0 : link.type) === null || _d === void 0 ? void 0 : _d.id) === global_ids_1.GLOBAL_ID_REJECTED)
                                    thenRejected = true;
                            }
                            debug('analized', { thenExists, thenResolved, thenRejected });
                            if (thenExists) {
                                if (thenResolved && !thenRejected) {
                                    debug('resolved');
                                    return res(true);
                                }
                                else if (thenRejected) {
                                    debug('rejected');
                                    return rej(false);
                                }
                                else {
                                    debug('waiting');
                                }
                            }
                            else {
                                debug('!then');
                                return res(true);
                            }
                        }
                        else {
                            debug('waiting');
                        }
                    }
                }
                catch (error) {
                    debug('error', error);
                    return rej(false);
                }
                yield (0, exports.delay)(timeout);
            }
        }
        catch (error) {
            debug('error', error);
            return rej(false);
        }
    }));
}
exports.awaitPromise = awaitPromise;
;
function findPromiseLink(options) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        const result = yield options.client.query((0, gql_1.generateQuery)({
            queries: [
                (0, gql_1.generateQueryData)({ tableName: 'links', returning: `id`, variables: { where: {
                            type_id: { _eq: global_ids_1.GLOBAL_ID_PROMISE },
                            in: {
                                type_id: { _eq: global_ids_1.GLOBAL_ID_THEN },
                                from_id: { _eq: options.id },
                            },
                        } } }),
            ],
            name: 'FIND_PROMISE',
        }));
        return (_b = (_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.q0) === null || _b === void 0 ? void 0 : _b[0];
    });
}
exports.findPromiseLink = findPromiseLink;
function reject(options) {
    return __awaiter(this, void 0, void 0, function* () {
        debug('reject', options.id);
        const promise = yield findPromiseLink(options);
        if (promise) {
            debug('rejected', yield options.client.mutate((0, gql_1.generateSerial)({
                actions: [(0, gql_1.insertMutation)('links', { objects: { type_id: global_ids_1.GLOBAL_ID_REJECTED, from_id: promise.id, to_id: promise.id } })],
                name: 'REJECT',
            })));
            return true;
        }
        return false;
    });
}
exports.reject = reject;
function resolve(options) {
    return __awaiter(this, void 0, void 0, function* () {
        debug('resolve', options.id);
        const promise = yield findPromiseLink(options);
        if (promise) {
            debug('resolved', yield options.client.mutate((0, gql_1.generateSerial)({
                actions: [(0, gql_1.insertMutation)('links', { objects: { type_id: global_ids_1.GLOBAL_ID_RESOLVED, from_id: promise.id, to_id: promise.id } })],
                name: 'RESOLVE',
            })));
            return true;
        }
        return false;
    });
}
exports.resolve = resolve;
//# sourceMappingURL=promise.js.map